#ifndef SENTECSENSORS_H
#define SENTECSENSORS_H

#include <Arduino.h>
#include <SoftwareSerial.h>

class SentecSensorRS485 {
  public:
	byte address;
	uint8_t serialCommunicationControlPin = 19;
	SoftwareSerial *RS485;
	byte answerFrame[10];
	// TODO use valid flag to log None
	bool valid = false;
	SentecSensorRS485(SoftwareSerial *ser, byte add);
	SentecSensorRS485(SoftwareSerial *ser, byte add, uint8_t serialControlPin);

	void write(byte queryFrame[], int length);
	String getValueStr(float value);
	String getValueStr(int value);
	void queryAddress();
	void readRegister(int registerStartAddress);
	void readRegister(int registerStartAddress, int registerLength);
	void writeRegister(int registerAddress, int value);
	void setAddress(byte add);
	void resetAnswerFrame();
	bool getResponse();
	unsigned int calculateCRC(byte query[], int length);
	void printBytes(byte *data, int length);
	void printBytes(word data);
};

class SolarRadiationSensor : public SentecSensorRS485 {
  public:
	using SentecSensorRS485::SentecSensorRS485;
	// global radiation [W/m^2]
	word glob = 0;

	word getSolarRadiation();
	String getSolarRadiationStr();
};

class RainGaugeSensor : public SentecSensorRS485 {
  public:
	using SentecSensorRS485::SentecSensorRS485;
	// precipitation values [mm]
	word precipitation = 0; // prcp since reset? (I THINK!!!)

	void resetPrecipitation();
	void resetSensor();
	word getInstantaneousPrecipitation();
	String getPrecipitationStr();
};

class SoilMoistureSensor : public SentecSensorRS485 {
  public:
	using SentecSensorRS485::SentecSensorRS485;
	// vwc: volumetric water content [%]
	uint16_t moistureRaw = 0;
	int moistureOffset = 0;
	// soil temperature [deg C]
	int temperatureRaw = 0;
	int temperatureOffset = 0;

	float getMoistureTemp();
	float getMoisture();
	String getMoistureStr();
	String getTemperatureStr();
};

#endif
