#ifndef _DRS26
#define _DRS26

#include "ForteSensor.hpp"
#include "Message.hpp"
#include "Pinout.hpp"
#include "Wire.h"
#include "esp_log.h"
#include <SDI12.h>
#include <map>

struct out_data_drs26 {
	int id;
	float circumferenceIncrement;
	float temperature;
};

class ForteDRS26 : public ForteSensor<out_data_drs26> {
  public:
	void setup() override;
	out_data_drs26 readData() override;
	std::list<Message> buildMessages() override;
	[[nodiscard]] SensorInformation getSensorInformation() const override;

  private:
	SDI12 drs26;
	out_data_drs26 data;
	const SensorInformation sensorInformation{"DRS26", Protocol::I2C};
	enum class MeasurementType { TEMPERATURE, CIRCUMFERENCE_INCREMENT };

	// enum to string
	std::map<MeasurementType, const char *> measurementTypeToString = {
	    {MeasurementType::TEMPERATURE, "TEMPERATURE"},
	    {MeasurementType::CIRCUMFERENCE_INCREMENT, "CIRCUMFERENCE_INCREMENT"}};
};

#endif