#ifndef _INA219
#define _INA219

#include "ForteSensor.hpp"
#include "Message.hpp"
#include "Pinout.hpp"
#include "Wire.h"
#include "esp_log.h"
#include <INA219_WE.h>

struct out_data_ina219 {
	float shuntVoltage_mV = 0.0;
	float loadVoltage_V = 0.0;
	float busVoltage_V = 0.0;
	float current_mA = 0.0;
	float power_mW = 0.0;
	bool ina219_overflow = false;
};

class ForteINA219 : public ForteSensor<out_data_ina219> {
  public:
	void setup() override;
	out_data_ina219 readData() override;
	std::list<Message> buildMessages() override;
	[[nodiscard]] SensorInformation getSensorInformation() const override;

  private:
	INA219_WE ina219;
	out_data_ina219 data;
	const SensorInformation sensorInformation{"INA219", Protocol::I2C};
	enum class MeasurementType {SHUNT_VOLTAGE, BUS_VOLTAGE, CURRENT_mA, POWER_mA, LOAD_VOLTAGE_V, INA219_OVERFLOW};

	std::map<MeasurementType, const char*> measurementTypeToString = {
		{MeasurementType::SHUNT_VOLTAGE, "SHUNT_VOLTAGE"},
		{MeasurementType::BUS_VOLTAGE, "BUS_VOLTAGE"},
		{MeasurementType::CURRENT_mA, "CURRENT_mA"},
		{MeasurementType::POWER_mA, "POWER_mA"},
		{MeasurementType::LOAD_VOLTAGE_V, "LOAD_VOLTAGE_V"},
		{MeasurementType::INA219_OVERFLOW, "INA219_OVERFLOW"}};
};

#endif