#ifndef _RS485
#define _RS485

#include "SPI.h"
// RTC (I2C)
#include "RTClib.h"
#include "Message.hpp"
#include "ForteSensor.hpp"
#include "SentecSensors.h"

struct out_data_rs485{
    float solarRadiation;
    float soilMoisture3;
    float soilTemperature3;
    float soilMoisture4;
    float soilTemperature4;
    float soilMoisture5;
    float soilTemperature5;
    float precipitation;
};

class Forte_RS485 : public ForteSensor <out_data_rs485> {
  public:
	void setup() override;
	out_data_rs485 readData() override;
	std::list<Message> buildMessages() override;
	[[nodiscard]] SensorInformation getSensorInformation() const override;

  private:
    const SensorInformation sensorInformation{"RS485", Protocol::RS485};

    enum class MeasurementType { 
        SOLAR_RADIATION,
        SOIL_MOISTURE_3,
        SOIL_TEMPERATURE_3,
        SOIL_MOISTURE_4,
        SOIL_TEMPERATURE_4,
        SOIL_MOISTURE_5,
        SOIL_TEMPERATURE_5,
        PRECIPITATION
      };

    std::map<MeasurementType, const char *> measurementTypeToString = {
	    {MeasurementType::SOLAR_RADIATION, "SOLAR_RADIATION"},
	    {MeasurementType::SOIL_MOISTURE_3, "SOIL_MOISTURE_3"},
      {MeasurementType::SOIL_TEMPERATURE_3, "SOIL_TEMPERATURE_3"},
      {MeasurementType::SOIL_MOISTURE_4, "SOIL_MOISTURE_4"},
      {MeasurementType::SOIL_TEMPERATURE_4, "SOIL_TEMPERATURE_4"},
	    {MeasurementType::SOIL_MOISTURE_5, "SOIL_MOISTURE_5"},
      {MeasurementType::SOIL_TEMPERATURE_5, "SOIL_TEMPERATURE_5"},
      {MeasurementType::PRECIPITATION, "PRECIPATION"}
        };
};
#endif 